# vue-draggable-nested-tree <a name="vue_draggable_nested_tree"></a>
这是可拖拽树组件. 此组件没有css, 您需要自己添加您喜欢的样式, 参考demo, 只有几个样式, 不难.
此组件不负责节点的具体渲染, 暴露了一个节点渲染插槽, 请参考demo自行渲染.   
This is a draggable tree component. This component does not have css, you need to add your style refer to demo. The demo style is less, not difficult.
This component doesn't render node. It exposes a node rendering slot. Please refer to the demo for rendering.   
[demo](https://codepen.io/phphe/pen/KRapQm)
# Indexes
* [vue-draggable-nested-tree](#vue_draggable_nested_tree)
* [install](#install)
* [usage](#usage)
  * [import](#import)
  * [data](#data)
  * [template](#template)
* [api](#api)
  * [Tree props](#tree_props)
     * [Noraml - Tree props](#noraml_tree_props)
     * [Hooks - Tree props](#hooks_tree_props)
     * [draggableHelperInfo: {event, options, store}](#draggable_helper_info:_{event,_options,_store})
  * [Tree properties](#tree_properties)
  * [Tree events](#tree_events)
  * [Tree methods](#tree_methods)
  * [node properties](#node_properties)
   * [node deep properties example](#node_deep_properties_example)
* [other](#other)
  * [demo css](#demo_css)
  * [examples](#examples)
  * [draggable & droppable](#draggable_&_droppable)
  * [Traverse tree](#traverse_tree)
  * [draggable library](#draggable_library)

# install <a name="install"></a>
```sh
npm i vue-draggable-nested-tree
```
# usage <a name="usage"></a>
### import <a name="import"></a>
```js
import {DraggableTree} from 'vue-draggable-nested-tree'
// vue-draggable-nested-tree contains Tree, TreeNode, DraggableTree, DraggableTreeNode
// import the component and register it as global or local component
```
### data <a name="data"></a>
```js
data: [
  {text: 'node 1'},
  {text: 'node 2'},
  {text: 'node 3 undraggable', draggable: false},
  {text: 'node 4'},
  {text: 'node 4 undroppable', droppable: false},
  {text: 'node 5', children: [
    {text: 'node 1'},
    {text: 'node 2', children: [
      {text: 'node 3'},
      {text: 'node 4'},
    ]},
    {text: 'node 2 undroppable', droppable: false, children: [
      {text: 'node 3'},
      {text: 'node 4'},
    ]},
    {text: 'node 2', children: [
      {text: 'node 3'},
      {text: 'node 4 undroppable', droppable: false},
    ]},
    {text: 'node 3'},
    {text: 'node 4'},
    {text: 'node 3'},
    {text: 'node 4'},
    {text: 'node 3'},
    {text: 'node 4'},
    {text: 'node 3'},
    {text: 'node 4'},
  ]},
]
```
### template <a name="template"></a>
```pug
Tree(:data="data" draggable crossTree)
  div(slot-scope="{data, store}")
    template(v-if="!data.isDragPlaceHolder")
      b(v-if="data.children && data.children.length" @click="store.toggleOpen(data)") {{data.open ? '-' : '+'}}&nbsp;
      span {{data.text}}
```
# api <a name="api"></a>
**The 'store' is the tree vm**
### Tree props <a name="tree_props"></a>
###### Noraml - Tree props <a name="noraml_tree_props"></a>
```js
// base tree
data: {},
indent: {default: 16},
activatedClass: {default: 'active'},
openedClass: {default: 'open'},
space: {default: 10}, // space between node, unit px
// draggable tree
getTriggerEl: {type: Function}, // get the el trigger drag, default is node self. arguments(nodeVm)
draggable: {}, // is the tree draggable, default false
droppable: {default: true}, // is the tree droppable, default true
crossTree: {}, // can a node of the tree be dragged into other tree, or receive other tree node
```
###### Hooks - Tree props <a name="hooks_tree_props"></a>
```js
ondragstart: {type: Function}, // hook. return false to prevent drag. arguments(node, draggableHelperInfo)
ondragend: {type: Function}, // hook. return false to prevent drop. arguments(node, draggableHelperInfo)
```
###### draggableHelperInfo: {event, options, store} <a name="draggable_helper_info:_{event,_options,_store}"></a>
### Tree properties <a name="tree_properties"></a>
```js
// base
rootData, // generated by tree
// draggable
dplh, // drag placeholder. globally unique.
trees, // array, all trees in the app. globally unique.
```
### Tree events <a name="tree_events"></a>
```js
// store is the tree vm
drag(node), // on drag start.
drop(node, targetTree, oldTree), // after drop.
change(node, targetTree, oldTree), // after drop, only when the node position changed
```
* targetTree and oldTree are tree vm.
* oldTree is available only when cross tree. Otherwise null.
* if cross tree, both targetTree and oldTree will emit drop and change
 targetTree and oldTree.

### Tree methods <a name="tree_methods"></a>
```js
pure(node, withChildren) // return a node data without runtime properties.(!: property which starts with '_' will be removed)
getNodeById(id)
getActivated()
getOpened()
activeNode(node, inactiveOld)
toggleActive(node, inactiveOld)
openNode(node, closeOld)
toggleOpen(node, closeOld)
// follow methods are easy, so I paste their soure code
getPureData() { return this.pure(this.rootData, true).children }
deleteNode(node) { return hp.arrayRemove(node.parent.children, node) }
// add node: like array. eg: node.children.push(newNodeData)
// update node: just assign to the node properties directly
```
### node properties <a name="node_properties"></a>
```js
// base
_id
_vm
parent
children: [],
open,
level,
active: false,
style: {},
class: '',
innerStyle: {},
innerClass: '',
innerBackStyle: {},
innerBackClass: {},
// draggable
draggable // default true. Please check 'draggable & droppable' below
droppable // default true. Please check 'draggable & droppable' below
isDragPlaceHolder
```
#### node deep properties example <a name="node_deep_properties_example"></a>
```js
node._vm // vm
node._vm.store // tree
node.parent._vm // parent node vm
node._vm.store.
```
# other <a name="other"></a>
### demo css <a name="demo_css"></a>
```css
.he-tree{
  border: 1px solid #ccc;
  padding: 20px;
  width: 300px;
}
.tree-node{
}
.tree-node-inner{
  padding: 5px;
  border: 1px solid #ccc;
  cursor: pointer;
}
.draggable-placeholder{
}
.draggable-placeholder-inner{
  border: 1px dashed #0088F8;
  box-sizing: border-box;
  background: rgba(0, 136, 249, 0.09);
  color: #0088f9;
  text-align: center;
  padding: 0;
  display: flex;
  align-items: center;
}
```
### examples <a name="examples"></a>
clone the package, and
```sh
npm install
npm run dev
```
* [Base](https://github.com/phphe/vue-draggable-nested-tree/blob/master/src/examples/Base.vue)
* [MaxLevel](https://github.com/phphe/vue-draggable-nested-tree/blob/master/src/examples/MaxLevel.vue)

### draggable & droppable <a name="draggable_&_droppable"></a>
A node is default draggable and droppable. You can set draggable and droppable property of a node. The another way is listen event 'drag', traverse all data to set draggable or droppable property.
### Traverse tree <a name="traverse_tree"></a>
Recommend to use my other library [tree-helper](https://github.com/phphe/tree-helper). It has 2 traverse methods: depthFirstSearch, breadthFirstSearch.
### draggable library <a name="draggable_library"></a>
[draggable-helper](https://github.com/phphe/draggable-helper) is my another library for drag. And it also is using by this component. You can use it to help you drag functions.
